@@ -351,7 +351,7 @@ EE flatten_call(EnvI& env, const Ctx& input_ctx, Expression* e, VarDecl* r, VarD
   FunctionI* decl = c->decl();
   try {
     decl = env.model->matchFn(env, c, false);
-  } catch (TypeError&) {
+  } catch (TypeError&) { /* NOLINT(bugprone-empty-catch) */
     // Can't actually match call, maybe it's being called with args which are now known to be
     // bottom, so just use the decl that was already in the call if there was one
   }
@@ -970,7 +970,7 @@ EE flatten_call(EnvI& env, const Ctx& input_ctx, Expression* e, VarDecl* r, VarD
           if (cr_d != nullptr) {
             decl = cr_d;
           }
-        } catch (TypeError&) {
+        } catch (TypeError&) { /* NOLINT(bugprone-empty-catch) */
           // Matches multiple incompatible overloads (e.g. because arg is actually bottom)
           // So just use original decl from type checker
         }
@@ -1076,7 +1076,7 @@ EE flatten_call(EnvI& env, const Ctx& input_ctx, Expression* e, VarDecl* r, VarD
         if ((reif_decl != nullptr) && (reif_decl->e() != nullptr)) {
           add_path_annotation(env, reif_decl->e());
           VarDecl* reif_b;
-          if (r == nullptr || (r != nullptr && r->e() != nullptr)) {
+          if (r == nullptr || (r != nullptr && r->e() != nullptr) || ctx.neg) {
             reif_b = new_vardecl(env, Ctx(), new TypeInst(Location().introduce(), Type::varbool()),
                                  nullptr, nullptr, nullptr);
             env.addCtxAnn(reif_b, ctx.b);
@@ -1108,7 +1108,7 @@ EE flatten_call(EnvI& env, const Ctx& input_ctx, Expression* e, VarDecl* r, VarD
           // Annotate cr() with get_path()
           add_path_annotation(env, cr());
           reif_b->e(cr());
-          if (r != nullptr && r->e() != nullptr) {
+          if (r != nullptr && r != reif_b) {
             Ctx reif_ctx;
             reif_ctx.neg = ctx.neg;
             bind(env, reif_ctx, r, reif_b->id());
